package eu.ztsh.lib.time;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.concurrent.TimeUnit;

import static eu.ztsh.lib.time.Formatters.hourMinutesShort;

/**
 * Work time diff counter.
 *
 * <p> Can be used for:
 * <ul>
 *     <li>days</li>
 *     <ul>
 *         <li>daysTillToday: 1</li>
 *         <li>secondsSpent: amount of work done on that day (in seconds)</li>
 *     </ul>
 *     <li>weeks</li>
 *     <ul>
 *         <li>daysTillToday: 5</li>
 *         <li>secondsSpent: amount of work done on that week (in seconds)</li>
 *     </ul>
 *     <li>any other period of time</li>
 * </ul>
 *
 * @param value String representation of difference between expected and real time
 * @param state Diff placement on timeline
 */
public record Diff(String value, State state) {

    public static final Diff OFF = new Diff("0:00", State.OFF);

    public static Diff calculate(long daysTillToday, long secondsSpent) {
        var totalSeconds = TimeUnit.HOURS.toSeconds(daysTillToday * 8);
        var diff = secondsSpent - totalSeconds;
        if (diff == 0) {
            return new Diff("0:00", State.EQUAL);
        }
        return diff > 0
                ? new Diff("+" + todayWithSeconds(diff).format(hourMinutesShort), State.ADVANCED)
                : new Diff("-" + todayWithSeconds(-diff).format(hourMinutesShort), State.OVERDUE);
    }

    private static LocalDateTime todayWithSeconds(long seconds) {
        return LocalDate.now().atTime(0, 0).plusSeconds(seconds);
    }

}
