package eu.ztsh.lib.time;

import jakarta.validation.constraints.NotNull;

import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.Objects;

import static eu.ztsh.lib.time.Formatters.hourMinutes;

/**
 * Object representation od elapsed time
 */
public class Duration implements Comparable<Duration> {

    private final LocalDateTime started;
    private final int timeSpentSeconds;
    private final LocalDateTime finished;

    public static Duration between(Duration previous, Duration next) {
        if (previous.started.equals(next.started)) {
            return new Duration(previous.started, 0, next.started);
        }
        var seconds = secondsBetween(previous.finished, next.started);
        return seconds < 0
                ? new Duration(next.finished, seconds, previous.started)
                : new Duration(previous.finished, seconds, next.started);
    }

    Duration(LocalDateTime started, LocalDateTime finished) {
        this.started = started;
        this.finished = finished;
        this.timeSpentSeconds = secondsBetween(started, finished);
    }

    Duration(LocalDateTime started, int timeSpentSeconds, LocalDateTime finished) {
        this.started = started;
        this.timeSpentSeconds = timeSpentSeconds;
        this.finished = finished;
    }

    public LocalDateTime getStarted() {
        return started;
    }

    public int getTimeSpentSeconds() {
        return timeSpentSeconds;
    }

    public LocalDateTime getFinished() {
        return finished;
    }

    /**
     * long -> gap -> short
     *
     * @param duration the object to be compared.
     * @return comparison result
     */
    @Override
    public int compareTo(@NotNull Duration duration) {
        var dates = started.compareTo(duration.started);
        return dates != 0 ? dates : duration.timeSpentSeconds - timeSpentSeconds;
    }

    @Override
    public String toString() {
        return "Duration(" + hourMinutes.format(started) + "-" + hourMinutes.format(finished) + ")";
    }

    @Override
    public int hashCode() {
        return Objects.hash(started, finished);
    }

    @Override
    public boolean equals(Object obj) {
        return (obj instanceof Duration duration)
                && duration.started.equals(this.started)
                && duration.finished.equals(this.finished);
    }

    private static int secondsBetween(LocalDateTime from, LocalDateTime to) {
        return Math.toIntExact(from.until(to, ChronoUnit.SECONDS));
    }

}
