package eu.ztsh.lib.time;

import org.junit.jupiter.api.Test;

import java.time.LocalDate;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatThrownBy;

class TimeRangeTest {

    private final LocalDate date1 = LocalDate.of(2023, 10, 10);
    private final LocalDate date2 = LocalDate.of(2023, 10, 12);
    private final LocalDate date3 = LocalDate.of(2023, 10, 13);
    private final LocalDate date4 = LocalDate.of(2023, 10, 15);

    @Test
    public void lengthTest() {
        var range = new TimeRange(date1, date2);
        assertThat(range.getDates()).hasSize(3);
        assertThat(range.getLength()).isEqualTo(3);
    }

    @Test
    public void equalityTest() {
        var range1 = new TimeRange(date1, date2);
        var range2 = new TimeRange(date1, date2);
        assertThat(range1).isEqualTo(range2);
    }

    @Test
    public void nonEqualityTest() {
        var range1 = new TimeRange(date1, date2);
        var range2 = new TimeRange(date1, date3);
        assertThat(range1).isNotEqualTo(range2);
    }

    @Test
    public void expandAfterTest() {
        var test = new TimeRange(date1, date2);
        var expected = new TimeRange(date1, date3);
        var result = test.expand(date3);
        assertThat(result).isEqualTo(expected);
    }

    @Test
    public void expandBeforeTest() {
        var test = new TimeRange(date2, date3);
        var expected = new TimeRange(date1, date3);
        var result = test.expand(date1);
        assertThat(result).isEqualTo(expected);
    }

    @Test
    public void expandInternalTest() {
        var test = new TimeRange(date1, date3);
        var expected = new TimeRange(date1, date3);
        var result = test.expand(date2);
        assertThat(result).isEqualTo(expected);
    }

    @Test
    public void mergeAfterTest() {
        var test1 = new TimeRange(date1, date2);
        var test2 = new TimeRange(date3, date4);
        var expected = new TimeRange(date1, date4);
        var result = test1.merge(test2);
        assertThat(result).isEqualTo(expected);
    }

    @Test
    public void mergeBeforeTest() {
        var test1 = new TimeRange(date1, date2);
        var test2 = new TimeRange(date3, date4);
        var expected = new TimeRange(date1, date4);
        var result = test2.merge(test1);
        assertThat(result).isEqualTo(expected);
    }

    @Test
    public void mergeIntersectingBeforeTest() {
        var test1 = new TimeRange(date1, date3);
        var test2 = new TimeRange(date2, date4);
        var expected = new TimeRange(date1, date4);
        var result = test1.merge(test2);
        assertThat(result).isEqualTo(expected);
    }

    @Test
    public void mergeIntersectionAfterTest() {
        var test1 = new TimeRange(date1, date3);
        var test2 = new TimeRange(date2, date4);
        var expected = new TimeRange(date1, date4);
        var result = test2.merge(test1);
        assertThat(result).isEqualTo(expected);
    }

    @Test
    public void mergeMinorContainingTest() {
        var test1 = new TimeRange(date1, date4);
        var test2 = new TimeRange(date2, date3);
        var expected = new TimeRange(date1, date4);
        var result = test1.merge(test2);
        assertThat(result).isEqualTo(expected);
    }

    @Test
    public void mergeMajorContainingTest() {
        var test1 = new TimeRange(date1, date4);
        var test2 = new TimeRange(date2, date3);
        var expected = new TimeRange(date1, date4);
        var result = test2.merge(test1);
        assertThat(result).isEqualTo(expected);
    }

    @Test
    public void mergeAdjacentTest() {
        var test1 = new TimeRange(date1, date2);
        var test2 = new TimeRange(date4, date4);
        assertThatThrownBy(() -> test1.merge(test2)).isInstanceOf(IllegalArgumentException.class);
    }

}